From 4dc28f3a14899d15d9acae11353f0f97bb81fa35 Mon Sep 17 00:00:00 2001
From: Pierre d'Herbemont <pdherbemont@free.fr>
Date: Wed, 16 Sep 2009 09:50:40 +0200
Subject: [PATCH 3/3] objects: Don't allow direct use of vlc_object_(release|hold) on vlc_object_t subclass without a warning.

This is something that work on a subclass basis.

This makes sure there is a warning when an invalid usage is detected.

Subclasses (AFAIK input and audio_instance) that implement refcounting should expose a hold() and a release(). They would simply need to call vlc_object_(release|hold)(VLC_OBJECT(a));.

Subclasses that does not implement refcounting should have a destructor paired with their constructor (eg CreateDecoder/DeleteDecoder). The destructor will call vlc_object_release(VLC_OBJECT(a));
---
 include/vlc_objects.h |   14 ++++++--------
 src/libvlccore.sym    |    4 ++--
 src/misc/objects.c    |    4 ++--
 3 files changed, 10 insertions(+), 12 deletions(-)

diff --git a/include/vlc_objects.h b/include/vlc_objects.h
index a5a692f..7345a13 100644
--- a/include/vlc_objects.h
+++ b/include/vlc_objects.h
@@ -79,8 +79,12 @@ VLC_EXPORT( void *, __vlc_object_find, ( vlc_object_t *, int, int ) );
 __attribute__((deprecated))
 #endif
 VLC_EXPORT( vlc_object_t *, vlc_object_find_name, ( vlc_object_t *, const char *, int ) );
-VLC_EXPORT( void *, __vlc_object_hold, ( vlc_object_t * ) );
-VLC_EXPORT( void, __vlc_object_release, ( vlc_object_t * ) );
+
+/* vlc_object_t are refcounted but only for its vlc_object_t members.
+ * Subclass may or may not be refcounted. */
+VLC_EXPORT( vlc_object_t *, vlc_object_hold, ( vlc_object_t * ) );
+VLC_EXPORT( void, vlc_object_release, ( vlc_object_t * ) );
+
 VLC_EXPORT( vlc_list_t *, __vlc_list_children, ( vlc_object_t * ) );
 VLC_EXPORT( void, vlc_list_release, ( vlc_list_t * ) );
 VLC_EXPORT( const char *, vlc_object_get_name, ( const vlc_object_t * ) ) LIBVLC_USED;
@@ -106,12 +110,6 @@ VLC_EXPORT( const char *, vlc_object_get_name, ( const vlc_object_t * ) ) LIBVLC
 #define vlc_object_find_name(a,b,c) \
     vlc_object_find_name( VLC_OBJECT(a),b,c)
 
-#define vlc_object_hold(a) \
-    __vlc_object_hold( VLC_OBJECT(a) )
-
-#define vlc_object_release(a) \
-    __vlc_object_release( VLC_OBJECT(a) )
-
 #define vlc_list_children(a) \
     __vlc_list_children( VLC_OBJECT(a) )
 
diff --git a/src/libvlccore.sym b/src/libvlccore.sym
index da2f4b6..ea81153 100644
--- a/src/libvlccore.sym
+++ b/src/libvlccore.sym
@@ -513,9 +513,9 @@ __vlc_object_create
 __vlc_object_detach
 __vlc_object_find
 vlc_object_find_name
-__vlc_object_hold
+vlc_object_hold
 __vlc_object_kill
-__vlc_object_release
+vlc_object_release
 __vlc_object_set_destructor
 vlc_object_get_name
 vlc_plugin_set
diff --git a/src/misc/objects.c b/src/misc/objects.c
index cb74287..5a3dee0 100644
--- a/src/misc/objects.c
+++ b/src/misc/objects.c
@@ -549,7 +549,7 @@ vlc_object_t *vlc_object_find_name( vlc_object_t *p_this,
 /**
  * Increment an object reference counter.
  */
-void * __vlc_object_hold( vlc_object_t *p_this )
+vlc_object_t * vlc_object_hold( vlc_object_t *p_this )
 {
     vlc_object_internals_t *internals = vlc_internals( p_this );
 
@@ -566,7 +566,7 @@ void * __vlc_object_hold( vlc_object_t *p_this )
  * Decrement an object refcount
  * And destroy the object if its refcount reach zero.
  *****************************************************************************/
-void __vlc_object_release( vlc_object_t *p_this )
+void vlc_object_release( vlc_object_t *p_this )
 {
     vlc_object_internals_t *internals = vlc_internals( p_this );
     vlc_object_t *parent = NULL;
-- 
1.6.4

